/*
 * Copyright (C) 2004-2012 Geometer Plus <contact@geometerplus.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef __ZLTQTAPPLICATIONWINDOW_H__
#define __ZLTQTAPPLICATIONWINDOW_H__

#include <map>

#include <tqmainwindow.h>
#include <tqtoolbutton.h>
#include <tqcursor.h>
#include <tqlineedit.h>

#include "ZLDesktopApplicationWindow.h"

class ZLTQtApplicationWindow : public TQMainWindow, public ZLDesktopApplicationWindow {
	TQ_OBJECT

public:
	ZLTQtApplicationWindow(ZLApplication *application);
	~ZLTQtApplicationWindow();

	void setFocusToMainWidget();

private:
	ZLViewWidget *createViewWidget();
	void addToolbarItem(ZLToolbar::ItemPtr item);
	void init();
	void processAllEvents();
	void close();

	void grabAllKeys(bool grab);

	void setCaption(const std::string &caption);

	void setHyperlinkCursor(bool hyperlink);

	bool isFullscreen() const;
	void setFullscreen(bool fullscreen);

	void closeEvent(TQCloseEvent *event);
	void keyPressEvent(TQKeyEvent *event);
	void wheelEvent(TQWheelEvent *event);

	void setToggleButtonState(const ZLToolbar::ToggleButtonItem &button);
	void setToolbarItemState(ZLToolbar::ItemPtr item, bool visible, bool enabled);

private:
	class TQToolBar *myToolBar;

friend class ZLTQtToolButton;
	std::map<const ZLToolbar::Item*,TQWidget*> myItemToWidgetMap;
	std::map<const ZLToolbar::MenuButtonItem*,size_t> myPopupIdMap;

	bool myFullScreen;
	bool myWasMaximized;

	bool myCursorIsHyperlink;
	TQCursor myStoredCursor;

friend class ZLTQtViewWidgetPositionInfo;

private:
	class LineEditParameter : public TQLineEdit, public VisualParameter {

	public:
		LineEditParameter(TQToolBar *toolbar, ZLTQtApplicationWindow &window, const ZLToolbar::ParameterItem &textFieldItem);

	private:
		void keyReleaseEvent(TQKeyEvent *event);

		std::string internalValue() const;
		void internalSetValue(const std::string &value);
		void setValueList(const std::vector<std::string> &values) {}

	private:
		ZLTQtApplicationWindow &myWindow;
		const std::string &myActionId;
	};
};

class ZLTQtToolButton : public TQToolButton {
	TQ_OBJECT

public:
	ZLTQtToolButton(ZLTQtApplicationWindow &window, ZLToolbar::AbstractButtonItem &item);

private slots:
	void onActivated();

private:
	ZLTQtApplicationWindow &myWindow;
	ZLToolbar::AbstractButtonItem &myItem;
};

#endif /* __ZLTQTAPPLICATIONWINDOW_H__ */
